﻿using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Shapes;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Wpf;

namespace AutoOrientationPopup_Wpf
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }
        
        private void Map1_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;

            WorldMapKitWmsWpfOverlay worldOverlay = new WorldMapKitWmsWpfOverlay();
            Map1.Overlays.Add(worldOverlay);

            PopupOverlay popupOverlay = new PopupOverlay();
            Map1.Overlays.Add(popupOverlay);
            ShapeFileFeatureLayer majorCitiesShapeLayer = new ShapeFileFeatureLayer(@"AppData\MajorCities.shp");
            majorCitiesShapeLayer.Open();
            Collection<Feature> features = majorCitiesShapeLayer.QueryTools.GetAllFeatures(ReturningColumnsType.AllColumns);
            foreach (Feature feature in features)
            {
                AutoOrientationPopup popup = new AutoOrientationPopup(feature.GetShape().GetCenterPoint());
                popup.Content = feature.ColumnValues["AREANAME"];
                popupOverlay.Popups.Add(popup);
            }
            majorCitiesShapeLayer.Close();

            Map1.CurrentExtent = popupOverlay.GetBoundingBox();
            Map1.CurrentExtent = new RectangleShape(-131.2163785, 49.7685379375, -62.3101285, 13.6454910625);
            Map1.Refresh();
        }
    }

    public class AutoOrientationPopup : Popup
    {
        public AutoOrientationPopup(PointShape position)
            : base(position)
        { }



        protected override void OnPositionChanged(DependencyPropertyChangedEventArgs eventArgs)
        {
            base.OnPositionChanged(eventArgs);
            AdjustOrientation();
        }

        private void AdjustOrientation()
        {
            Polygon arrow = GetTemplateChild("ArrowTemplate") as Polygon;
            Border contentRectangle = GetTemplateChild("BorderTemplate") as Border;

            if (arrow != null && contentRectangle != null)
            {
                var popupPointX = (double)this.GetValue(Canvas.LeftProperty);
                var popupPointY = (double)this.GetValue(Canvas.TopProperty);
                contentRectangle.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));

                arrow.Points.Clear();
                arrow.Margin = new Thickness(0, 0, 0, contentRectangle.DesiredSize.Height * 0.5);
                if (popupPointY - contentRectangle.ActualHeight - ArrowHeight < 0)
                {
                    this.SetValue(Canvas.TopProperty, popupPointY + contentRectangle.ActualHeight + ArrowHeight);
                    arrow.Points.Add(new Point(-10, 0 - contentRectangle.ActualHeight));
                    arrow.Points.Add(new Point(10, 0 - contentRectangle.ActualHeight));
                    arrow.Points.Add(new Point(0, 0 - ArrowHeight - contentRectangle.ActualHeight));
                }
                else
                {
                    arrow.Points.Add(new Point(-10, 0));
                    arrow.Points.Add(new Point(10, 0));
                    arrow.Points.Add(new Point(0, ArrowHeight));
                }
            }
        }
    }
}